%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% COMPRESION DE VIDEO                                          %%
%% Grado en Ingeniera de Tecnologas de Telecomunicacin       %%
%% Gustavo A. Ruiz Robredo                                      %%
%% ruizrg@unican.es                                             %%
%%                                                              %%
%% uu   uu  cccccc   UNIVERSIDAD DE CANTABRIA                   %%
%% uu   uu  cccccc   Dpto. INGENIERIA INFORMATICA Y ELECTRONICA %%
%% uu   uu  cc       FACULTAD DE CIENCIAS                       %%
%%  uuuuu   cccccc   Avda. Los Castros s/n                      %%
%%   uuu    cccccc   39005-SANTANDER (SPAIN)                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% En este ejemplo se aplica el algoritmo de estimacin de movimiento full search a una
% frame
clear all;
close all;

% Caractersticas del video a procesar
high=288;
width=352;
numTotalFrames=21;
nameVideo='Vectra@21@352x288.yuv';

% Se leen los ficheros current y reference video
[Yvid, Uvid, Vvid]=leerVideoYUV_420(nameVideo,   high, width, numTotalFrames);

% Seleccin de frames  numReferenceFrame < numCurrentFrame
numReferenceFrame=1; % Se selecciona la reference frame
numCurrentFrame=2; % Se selecciona la current frame

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se va a visualizar current frame con una grid MB de 16x16
titulo=sprintf('CURRENT FRAME %d: size (%dx%d)',numCurrentFrame,width,high);
Ygrid=squeeze(Yvid(:,:,numCurrentFrame));
Ygrid(:,1:16:width)=255; % Grid horizontal
Ygrid(1:16:high,:)=255; % Grid vertical
Ugrid=squeeze(Uvid(:,:,numCurrentFrame));
Vgrid=squeeze(Uvid(:,:,numCurrentFrame));
visualizarFrameYUV_420( Ygrid,Ugrid,Vgrid,0, 0, high, width, titulo);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Area de busqueda (16+2*searchArea)*(16+2*searchArea)
searchArea=8;

% Extension de las lumas y cromas
for x=1:width+2*searchArea
    for y=1:high+2*searchArea
        if( y-searchArea <= 0 )
            posY=1;
        elseif( y-searchArea > high)
            posY=high;
        else
            posY=y-searchArea;
        end
        
        if( x-searchArea <= 0 )
            posX=1;
        elseif( x-searchArea > width)
            posX=width;
        else
            posX=x-searchArea;
        end
        
        % Asignacion de valores
        Yext(y,x)=Yvid(posY,posX,numReferenceFrame);
        Uext( floor((y+1)/2),floor((x+1)/2) )=Uvid(floor((posY+1)/2), floor((posX+1)/2),numReferenceFrame);
        Vext( floor((y+1)/2),floor((x+1)/2) )=Vvid(floor((posY+1)/2), floor((posX+1)/2),numReferenceFrame);
    end
end

% Visualizacion de la imagen extendida con searchArea
titulo=sprintf('EXTENDED REFERENCE FRAME %d: size (%dx%d)',numReferenceFrame, width+2*searchArea,high+2*searchArea);
visualizarFrameYUV_420( Yext,Uext,Vext,0, 0, high+2*searchArea, width+2*searchArea, titulo);

% Se selecciona cada uno de los macrobloques 16x16 actuales a procesar
% Algoritmo de Full Search
for MBx=0:(width/16)-1
    for MBy=0:(high/16)-1
        % Origen
        X=MBx*16;
        Y=MBy*16;
        minMVx=0;
        minMVy=0;
        
        fprintf(1,'\n MBx=%3d MBy=%3d',MBx, MBy);
        
        % Seleccion del current MB 16x16
        % +1 porque el ndice en matrices de matlab empieza en 1, no en 0
        YMBcur(1:16,1:16)=Yvid(Y+1:Y+16,X+1:X+16,numCurrentFrame);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Se calcula los diferentes costes SAD y el vector de movimiento en cada caso
        minSAD=65535; % Inicializado a infinito
        
        % Se recorre todos los bloques 16x16 dentro del rea de bsqueda
        for dx=-searchArea:searchArea
            for dy=-searchArea:searchArea
                % Asignacion de valor
                YRefsel(1:16,1:16)=Yext(Y+dy+searchArea+1:Y+dy+searchArea+16,X+dx+searchArea+1:X+dx+searchArea+16);
                
                % Clculo coste SAD
                SAD = costFuncSAD(YRefsel, YMBcur);
                
                % Se comprueba el minimo
                if (SAD < minSAD)
                    minSAD=SAD;
                    minMVx=dx;
                    minMVy=dy;
                end
            end
        end
        
        fprintf(1,' ... minMVx=%3d minMVy=%3d',minMVx, minMVy);
        
        % Se guarda la imagen con mejor match
        Yres(Y+1:Y+16,X+1:X+16)=Yext(Y+minMVy+searchArea+1:Y+minMVy+searchArea+16,X+minMVx+searchArea+1:X+minMVx+searchArea+16);
        Ures(floor((Y+1)/2)+1:floor((Y+1)/2)+8,floor((X+1)/2)+1:floor((X+1)/2)+8)=...
            Uext(floor((Y+minMVy+searchArea)/2)+1:floor((Y+minMVy+searchArea)/2)+8,...
            floor((X+minMVx+searchArea)/2)+1:floor((X+minMVx+searchArea)/2)+8);
        Vres(floor((Y+1)/2)+1:floor((Y+1)/2)+8,floor((X+1)/2)+1:floor((X+1)/2)+8)=...
            Vext(floor((Y+minMVy+searchArea)/2)+1:floor((Y+minMVy+searchArea)/2)+8,...
            floor((X+minMVx+searchArea)/2)+1:floor((X+minMVx+searchArea)/2)+8);
        
        % Se Almacena en un frame los vectores de movimiento
        if(minMVx ~= 0)
            for x=0:sign(minMVx):minMVx-sign(minMVx)
                if(minMVy ~= 0)
                    for y=round(x*minMVy/minMVx):sign(x*minMVy/minMVx):round((x+sign(minMVx))*minMVy/minMVx)
                        Ygrid(Y+9+y,X+9+x)=200;
                    end
                else
                    Ygrid(Y+9,X+9+x)=200;
                end
            end
        else
            if(minMVy ~= 0)
                for y=0:sign(minMVy):minMVy
                    Ygrid(Y+9+y,X+9)=200;
                end
            else
                Ygrid(Y+8,X+8)=200;
                Ygrid(Y+9,X+8)=200;
                Ygrid(Y+8,X+9)=200;
                Ygrid(Y+9,X+9)=200;
            end
        end
    end
end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %visualizacion de los vectores de movimiento
titulo=sprintf('CURRENT FRAME: Vectores de Mvto.');
visualizarFrameYUV_420(Ygrid, Ugrid, Vgrid, 0, 0, high, width, titulo);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %visualizacion del best matching
titulo=sprintf('BEST MATCHED');
visualizarFrameYUV_420( Yres, Ures, Vres,0,0,high,width,titulo);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ERROR entre la imagen Current e la imagen de mejor match
fprintf(1,'\n PSNR=%f\n', imgPSNR(Yres, squeeze( Yvid(:,:,numCurrentFrame)), 255) );
