%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% COMPRESION DE VIDEO                                          %%
%% Grado en Ingeniera de Tecnologas de Telecomunicacin       %%
%% Gustavo A. Ruiz Robredo                                      %%
%% ruizrg@unican.es                                             %%
%%                                                              %%
%% uu   uu  cccccc   UNIVERSIDAD DE CANTABRIA                   %%
%% uu   uu  cccccc   Dpto. INGENIERIA INFORMATICA Y ELECTRONICA %%
%% uu   uu  cc       FACULTAD DE CIENCIAS                       %%
%%  uuuuu   cccccc   Avda. Los Castros s/n                      %%
%%   uuu    cccccc   39005-SANTANDER (SPAIN)                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Este ejemplo permite visualizar un frame de un video en formato YUV 4:2:0
% El bloque procesado es de 16x16 con una funcin coste SAD
% Algoritmo full-search
clear all;
close all;

% seleccin de las imgenes a visualizar
numCurrentFrame=3; % Se selecciona la current frame
numRefFrame=1; % Seleccin de la imagen de refrencia

% Area de busqueda (16+2*searchArea)*(16+2*searchArea)
searchArea=16;

% Caractersticas del video a procesar
high=288;
width=352;
numTotalFrames=6;
nameVideo='Vectra@21@352x288.yuv';

% Se leen los ficheros current y reference video
[Ycur, Ucur, Vcur]=leerVideoYUV_420(nameVideo,   high, width, numTotalFrames);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se selecciona el origen del bloque actual a procesar
X=16*9; % 288
Y=16*10; % 240

% X=288;
% Y=240;

% Seleccion MB 16x16 de current frame
% +1 porque el ndice en matrices de matlab empieza en 1, no en 0
YC(1:16,1:16)=squeeze( Ycur(Y+1:Y+16,X+1:X+16,numCurrentFrame) );
UC=squeeze ( Ucur(floor(Y/2)+1:floor(Y/2)+8,floor(X/2)+1:floor(X/2)+8,numCurrentFrame));
VC=squeeze ( Vcur(floor(Y/2)+1:floor(Y/2)+8,floor(X/2)+1:floor(X/2)+8,numCurrentFrame));

% Se visualiza el bloque actual
titulo=sprintf('CURRENT FRAME %d: Bloque current POS (%d,%d) size (%dx%d)',numCurrentFrame,Y, X, 16, 16);
visualizarFrameYUV_420( squeeze(Ycur(:,:,numCurrentFrame)),...
    squeeze(Ucur(:,:,numCurrentFrame)),...
    squeeze(Vcur(:,:,numCurrentFrame)), Y, X, 16, 16, titulo);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se va a visualizar la reference frame



% Se visualiza el area de bsqueda
titulo=sprintf('REFERENCE FRAME %d: Area de busqueda POS (%d,%d) size (%dx%d)',...
    numRefFrame,X-searchArea, Y-searchArea, 2*searchArea+16, 2*searchArea+16);
visualizarFrameYUV_420(    squeeze(Ycur(:,:,numRefFrame)),...
    squeeze(Ucur(:,:,numRefFrame)),...
    squeeze(Vcur(:,:,numRefFrame)),...
    Y-searchArea, X-searchArea,  2*searchArea+16, 2*searchArea+16, titulo);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se calcula los diferentes costes SAD y el vector de movimiento en cada caso
minSAD=65535;
SAD=zeros(2*searchArea+1, 2*searchArea+1); % Reserva memoria

% Se recorre todos los bloques 16x16 dentro del rea de bsqueda
for dx=-searchArea:searchArea
    for dy=-searchArea:searchArea
        % Se selecciona un bloque 16x16 en el area de bsqueda
        YRsel(1:16,1:16)=Ycur(Y+dy+1:Y+dy+16,X+dx+1:X+dx+16,numRefFrame);
        
        % Se llama a la funcin coste SAD
        SAD(dx+searchArea+1,dy+searchArea+1) = costFuncSAD(YRsel, YC);
        
        % Se comprueba el minimo
        if (SAD(dx+searchArea+1,dy+searchArea+1) < minSAD)
            minSAD=SAD(dx+searchArea+1,dy+searchArea+1);
            minMVx=dx;
            minMVy=dy;
        end
    end
end

% Resultado del mejor estimacin de movimiento
fprintf(1,'\n BEST MATCH: REFERENCE FRAME=%2d CURRENT FRAME=%2d -> SAD=%5d MVx=%2d MVy=%2d\n',...
    numRefFrame, numCurrentFrame, minSAD,minMVx, minMVy);

% Visualizacin de bidimensional del SAD
figure
title('Valores de SAD')
xlabel('MVy')
ylabel('MVx')
zlabel('SAD')
searchX=-searchArea:searchArea;
searchY=-searchArea:searchArea;
surface(searchX,searchY,SAD);
view(-30,30);
grid on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%visualizacion del best matching o Block frame con minimo coste
titulo=sprintf('Mejor matching MVy=%d MVx=%d POS (%d,%d) size (%dx%d)',...
    minMVy, minMVx, Y+minMVy, X+minMVx, 16, 16);
visualizarFrameYUV_420(    squeeze(Ycur(:,:,numRefFrame)),...
    squeeze(Ucur(:,:,numRefFrame)), squeeze(Vcur(:,:,numRefFrame)),...
    Y+minMVy, X+minMVx,  16, 16,titulo);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compensacion
% Best match in la frame de refencia;
Ybest=squeeze ( Ycur(Y+minMVy+1:Y+minMVy+16,X+minMVx+1:X+minMVx+16,numRefFrame));
Ubest=squeeze ( Ucur(floor((Y+minMVy)/2)+1:floor((Y+minMVy)/2)+8,floor((X+minMVx)/2)+1:floor((X+minMVx)/2)+8,numRefFrame));
Vbest=squeeze ( Vcur(floor((Y+minMVy)/2)+1:floor((Y+minMVy)/2)+8,floor((X+minMVx)/2)+1:floor((X+minMVx)/2)+8,numRefFrame));

% Imagen compensada
fprintf(1,'\n IMAGEN COMPENSADA:');
Ycomp=YC-Ybest
Ucomp=Ubest-UC
Vcomp=Vbest-VC
fprintf(1,'\n');
