function visualizarFrameYUV_420 (Yluma, Ucroma, Vcroma, Yorig, Xorig, deltaY, deltaX, titulo);
% Funcion que permite visualizar un trozo o una frame entera en formato YUV 4:2:0 8 bits
% Argumentos de entrada:
%    Y: matriz luminancia
%    U: matriz de Croma
%    V: matriz de Croma
%    (Yorig, Xorig) : origen de coordenadas
%    (deltaY, deltaX) : tamao de la ventana
%     titulo: Texto para adir en el ttulo
% Argumentos de salida:

% Comprobacin de lmites
[N M]=size(Yluma);
if( (Xorig+deltaX > M) || (Yorig+deltaY > N) )
    fprintf(1,'\n ERROR: Superado dimensiones de la Frame(%d x %d) < (%d x %d)', N, M, Yorig+deltaY, Xorig+deltaX);
    beep
    return;
end


% Seleccion de la LUMA
YC(1:deltaY,1:deltaX)=uint8(Yluma(Yorig+1:Yorig+deltaY,Xorig+1:Xorig+deltaX));

% Extension de la CROMA a 4:4:4
CBC(1:2:deltaY,1:2:deltaX)=uint8( Ucroma(floor(Yorig/2)+1:floor((Yorig+deltaY)/2), floor(Xorig/2)+1:floor((Xorig+deltaX)/2)) );
CBC(1:2:deltaY,2:2:deltaX)=uint8( Ucroma(floor(Yorig/2)+1:floor((Yorig+deltaY)/2), floor(Xorig/2)+1:floor((Xorig+deltaX)/2)) );
CBC(2:2:deltaY,1:2:deltaX)=uint8( Ucroma(floor(Yorig/2)+1:floor((Yorig+deltaY)/2), floor(Xorig/2)+1:floor((Xorig+deltaX)/2)) );
CBC(2:2:deltaY,2:2:deltaX)=uint8( Ucroma(floor(Yorig/2)+1:floor((Yorig+deltaY)/2), floor(Xorig/2)+1:floor((Xorig+deltaX)/2)) );

CRC(1:2:deltaY,1:2:deltaX)=uint8( Vcroma(floor(Yorig/2)+1:floor((Yorig+deltaY)/2), floor(Xorig/2)+1:floor((Xorig+deltaX)/2)) );
CRC(1:2:deltaY,2:2:deltaX)=uint8( Vcroma(floor(Yorig/2)+1:floor((Yorig+deltaY)/2), floor(Xorig/2)+1:floor((Xorig+deltaX)/2)) );
CRC(2:2:deltaY,1:2:deltaX)=uint8( Vcroma(floor(Yorig/2)+1:floor((Yorig+deltaY)/2), floor(Xorig/2)+1:floor((Xorig+deltaX)/2)) );
CRC(2:2:deltaY,2:2:deltaX)=uint8( Vcroma(floor(Yorig/2)+1:floor((Yorig+deltaY)/2), floor(Xorig/2)+1:floor((Xorig+deltaX)/2)) );

% Se concatenan las matrices
ycbcrC=cat(3,YC, CBC, CRC);
% Se convierte a RGB
rgbC=ycbcr2rgb(ycbcrC);
% Se visualiza la current frame
figure
imshow(rgbC)
title(titulo);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% COMPRESION DE VIDEO                                          %%
%% Grado en Ingeniera de Tecnologas de Telecomunicacin       %%
%% Gustavo A. Ruiz Robredo                                      %%
%% ruizrg@unican.es                                             %%
%%                                                              %%
%% uu   uu  cccccc   UNIVERSIDAD DE CANTABRIA                   %%
%% uu   uu  cccccc   Dpto. INGENIERIA INFORMATICA Y ELECTRONICA %%
%% uu   uu  cc       FACULTAD DE CIENCIAS                       %%
%%  uuuuu   cccccc   Avda. Los Castros s/n                      %%
%%   uuu    cccccc   39005-SANTANDER (SPAIN)                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
